<?php
// $Id: regcode.api.php,v 1.1.2.2 2009/06/17 09:32:21 aidan Exp $

/**
 * Return text message requested by given identifier/constant
 *
 * @param $code
 *  The message code
 * @return
 *  The text of the message.
 */
function regcode_message($code) {
  $messages[REGCODE_VALIDITY_NOTEXISTING]  = 'Registration code does not exist.';
  $messages[REGCODE_VALIDITY_NOTAVAILABLE] = 'Registration code is not available yet.';
  $messages[REGCODE_VALIDITY_TAKEN]        = 'Registration code has already been used.';
  $messages[REGCODE_VALIDITY_EXPIRED]      = 'Registration code has expired.';
  return $messages[$code];
}


/**
 * Retrieve a particular registration code identified by given code id,
 * or assign it to a given user
 *
 * @param $code
 *  The code identifiert string
 * @param $assign_to_uid
 *  Optional user id to assign the given code to
 * @return
 *  The data array of the code to retrieve, or a boolean result (TRUE=successfully assigned to given uid) if assign_to_uid has been given.
 */
function regcode_get_code($code, $use_code = FALSE) {

  // look up and retrieve code record from DB
  $code = db_fetch_array(db_query("SELECT code, begins, expires, is_active, maxuses, uses FROM {regcode} WHERE code = '%s'", $code));

  // check validity
  if (!is_array($code)) {
    return REGCODE_VALIDITY_NOTEXISTING;
  }
  
  if (!$code['is_active']) {
    return REGCODE_VALIDITY_NOTAVAILABLE;
  }
  
  if (!empty($code['begins']) && strtotime($code['begins']) < time()) {
    return REGCODE_VALIDITY_NOTAVAILABLE;
  }
  
  if (!empty($code['expires']) && strtotime($code['expires']) > time()) {
    return REGCODE_VALIDITY_EXPIRED;
  }
  
  if ($code['uses'] >= $code['maxuses']) {
    return REGCODE_VALIDITY_TAKEN;
  }
  
  // optionally mark this code as taken for the given user uid
  if ($use_code) {
    drupal_set_message('updating' . $code['code']);
    db_query("UPDATE {regcode} SET uses=uses+1, lastused=NOW(), uid=%d WHERE code='%s'", $use_code, $code['code']);
  }

  return $code;
}


/**
 * Return the database query result for the given options to list codes
 *
 * @return
 *   An array list of code data arrays
 */
function regcode_get_codes($options = array()) {

  $result = db_query('SELECT rid, uid, created, lastused, begins, expires, category, code, is_active, maxuses, uses FROM {regcode}');
  
  $data = array();
  while ($row = db_fetch_array($result)) {
    $data[] = $row;
  }
  
  return $data;
}


/**
 * Save given code array to a record in the db
 *
 * @param $code
 *   An array of code data fields, containing the fields:
 * @param $action
 *   Action to perform when saving the code:
 *     overwrite  -  overwrite any existing code with same code identifier
 *     skip       -  skip and do not save code if there is any existing code with same code identifier
 *
 * @return
 *   An array list of code data arrays
 */
function regcode_save_code($code, $action = 'overwrite') {
  // Sanity check
  if (empty($code) || empty($code['code'])) {
    return FALSE;
  }
  
  // optionally delete old record
  if ($action == 'overwrite') {
    db_query('DELETE FROM {regcode} WHERE code = "%s" LIMIT 1', $code['code']);
  }
  
  // insert record into db
  $sql = 'INSERT INTO {regcode} (created,begins,expires,category,code,is_active,is_onetime)
          VALUES (NOW(), %s, %s, "%s", "%s", %d, %d)';
  $result = db_query($sql,
    empty($code['begins']) ? 'NULL' : '"' . $code['begins'] . '"',
    empty($code['expires']) ? 'NULL' : '"' . $code['expires'] . '"',
    $code['category'],
    $code['code'],
    $code['is_active'],
    $code['is_onetime']);
  return $result;
}


/**
 * Clean up codes
 *
 * @param $op
 *   Action to perform when cleaning the code:
 *     truncate      -  truncate all codes from db
 *     expired       -  remove expired codes
 *     used          -  remove one-time codes that have been used
 */
function regcode_clean_codes($op = 'truncate') {
  switch ($op) {
    case 'truncate':
      db_query('TRUNCATE {regcode}');
      break;
    
    case 'expired':
      db_query('DELETE FROM {regcode} WHERE expired < NOW()');
      break;
    
    case 'used':
      db_query('DELETE FROM {regcode} WHERE is_onetime=1 AND uses > 0');
      break;
  }
}


/**
 * Generate a code
 */
function regcode_generate_code($length, $output, $case) {
  static $seeded = false;
  
  // Possible seeds
  $outputs['alpha']    = 'abcdefghijklmnopqrstuvwqyz';
  $outputs['numeric']  = '0123456789';
  $outputs['alphanum'] = 'abcdefghijklmnopqrstuvwqyz0123456789';
  $outputs['hexadec']  = '0123456789abcdef';

  // Choose seed
  if (isset($outputs[$output])) {
    $output = $outputs[$output];
  }

  // Seed generator (only do this once per invocation)
  if (!$seeded) {
    list($usec, $sec) = explode(' ', microtime());
    $seed = (float) $sec + ((float) $usec * 100000);
    mt_srand($seed);
    $seeded = true;
  }
  
  // Generate
  $str = '';
  $output_count = strlen($output);
  for ($i = 0; $length > $i; $i++) {
    $str .= $output{mt_rand(0, $output_count - 1)};
  }

  if ($case) {
    $str = strtoupper($str);
  }
  
  return $str;
}


/**
 * Import codes from a file (plain list or CSV)
 *
 * @param $filepath
 *   File system path of the file to import from
 * @param $action
 *   Import action:
 *     overwrite  -  overwrite any existing codes with same code identifiers as the imported ones
 *     skip       -  skip any import of a codes, if any code with same code identifier exists
 *     clean      -  completely wipe the code database befor importing all codes
 * @param $code_template
 *   Optional code data template to use as default data setting for each imported code (key/value pairs for any applicable code field, see regcode_save_code)
 *
 * @return
 *   Whether the import operation has been successful (TRUE) or not (FALSE)
 */
function regcode_import_file($filepath, $action = NULL, $code_template = NULL) {

  // open file as stream and delegate to regcode_import_stream
  $handle = fopen($filepath, "r");
  $result = regcode_import_stream($handle, $action, $code_template);
  fclose($handle);

  return $result;
}


/**
 * Import codes from a multi-line string (plain list or CSV)
 *
 * @param $text
 *   Multi-line string to import from
 * @param $action
 *   Import action:
 *     overwrite  -  overwrite any existing codes with same code identifiers as the imported ones
 *     skip       -  skip any import of a codes, if any code with same code identifier exists
 *     clean      -  completely wipe the code database befor importing all codes
 * @param $code_template
 *   Optional code data template to use as default data setting for each imported code (key/value pairs for any applicable code field, see regcode_save_code)
 *
 * @return
 *   Whether the import operation has been successful (TRUE) or not (FALSE)
 */
function regcode_import_text(&$text, $action = NULL, $code_template = NULL) {

  // open string as memory stream and delegate to regcode_import_stream
  $handle = fopen("php://temp/maxmemory:". (5 * 1024 * 1024), 'rw');
  fputs($handle, $text);
  rewind($handle);
  $result = regcode_import_stream($handle, $action, $code_template);
  fclose($handle);

  return $result;
}


/**
 * Import codes from a stream (plain list or CSV text data)
 *
 * @param $stream
 *   Stream handle of the stream to import from
 * @param $action
 *   Import action:
 *     overwrite  -  overwrite any existing codes with same code identifiers as the imported ones
 *     skip       -  skip any import of a codes, if any code with same code identifier exists
 *     clean      -  completely wipe the code database befor importing all codes
 * @param $code_template
 *   Optional code data template to use as default data setting for each imported code (key/value pairs for any applicable code field, see regcode_save_code)
 *
 * @return
 *   Whether the import operation has been successful (TRUE) or not (FALSE)
 */
function regcode_import_stream($stream, $action = NULL, $code_template = NULL) {
  if (!$stream) return FALSE;
  if (!$action) $action = variable_get('regcode_import_action', 'skip');

  $delimiter  = variable_get('regcode_import_csv_delimiter', ',');
  $enclosure  = variable_get('regcode_import_csv_text_enclosure', '"');
  $fieldorder = variable_get('regcode_import_csv_fieldorder', 'code');

  // clean codes when needed (depending on action)
  regcode_clean_codes($action);

  // iterate through stream line by line
  while (($row = fgetcsv($stream, 9999, $delimiter, $enclosure)) !== FALSE) {
    if (empty($fieldorder)) {
      $fieldorder = $row;
    }
    else {
      if (regcode_import_code($row, $action, $fieldorder, $code_template)) $count++;
    }
  }

  drupal_set_message(t('!count registration codes imported.', array('!count' => intval($count))));
  watchdog('RegistrationCode', t('!count registration codes imported.', array('!count' => intval($count))));
  return ($count > 0);
}


/**
 * Import a single codes from a single ine string (plain one-field or CSV)
 *
 * @param $data
 *   The single-line string containing the code data to import (a single field or CSV)
 * @param $action
 *   Import action:
 *     overwrite  -  overwrite any existing code with same code identifier as the imported one
 *     skip       -  skip import of code, if any code with same code identifier exists
 *     *          -   "^
 * @param $fieldorder
 *   List of field keys indicating the order of field to expect in the $data string (array or delimiter-seperated string of keys)
 * @param $code
 *   Optional code data template to use as default data setting for each imported code (key/value pairs for any applicable code field, see regcode_save_code)
 *
 * @return
 *   Whether the import operation has been successful (TRUE) or not (FALSE)
 */
function regcode_import_code($data, $action, $fieldorder = NULL, $code = array()) {
  if (empty($fieldorder))     $fieldorder = variable_get('regcode_import_csv_fieldorder', 'code');
  if (!is_array($fieldorder)) $fieldorder = explode(variable_get('regcode_import_csv_delimiter', ','), $fieldorder);

  foreach ($data as $key => $value) $code[ $fieldorder[$key] ] = $value;

  return regcode_save_code($code, $action);
}
